
package fr.wikimedia.miniServer;

import java.util.*;
import java.io.*;
import java.util.zip.*;
import p7zip.LZMA.LzmaInputStream;

class ManyZipFileServer extends NanoHTTPD {
    String dirBase;
    ManyZipFileBuckets buckets;

    ManyZipFileServer(String zi7Base, String dirBase, int nrBuckets, int port) throws IOException {
	super(port);
	buckets = new ManyZipFileBuckets(zi7Base, nrBuckets);
	this.dirBase = dirBase;
    }

    public Response serve( String uri, String method, Properties header, Properties parms) {
	/*
	  Enumeration e = header.propertyNames();
	  while ( e.hasMoreElements()) {
	  String value = (String) e.nextElement();
	  System.out.println( "  HDR: '" + value + "' = '" +
	  header.getProperty( value ) + "'" );
	  }
	  e = parms.propertyNames();
	  while ( e.hasMoreElements()) {
	  String value = (String) e.nextElement();
	  System.out.println( "  PRM: '" + value + "' = '" +
	  parms.getProperty( value ) + "'" );
	  }
	*/
	if (uri.length() <= 1 || uri.charAt(0)!='/') {
	    return new Response( HTTP_NOTFOUND, MIME_PLAINTEXT,
				 "Error 404, bad file format: " + uri);
	}
	String filename = uri.substring(1);

	if (filename.indexOf("../") > 0) {
          return new Response( HTTP_NOTFOUND, MIME_PLAINTEXT,
                                         "Error 404, filename contains ../" );
        }

	InputStream iStream=null;
	long length= 0;
	
	try {
	    File file = new File(dirBase, filename.replace('/', File.separatorChar));
	    // System.out.println("local file: " + file);
	    iStream = new FileInputStream(file);
	    length = file.length();
	} catch (IOException exn) {
	    String bucketFilename = buckets.getCompressedStorageName(filename);
	    try {
		if (! buckets.isInIndex(filename)) {
		    return new Response( HTTP_NOTFOUND, MIME_PLAINTEXT,
					 "Error 404, not in filelist: " + filename);
		}

		FileInputStream zi7File = new FileInputStream(bucketFilename);

		if (zi7File == null) {
		    return new Response( HTTP_NOTFOUND, MIME_PLAINTEXT,
					 "Error 404, zi7 file not found: " + bucketFilename);
		}
		
		ZipInputStream zipStream = new ZipInputStream(new LzmaInputStream(zi7File));
		
		while (zipStream.available() > 0) {
		    ZipEntry entry = zipStream.getNextEntry();
		    if (entry == null) break;
		    if (entry.getName().equals(filename)) {
			//System.out.println("entry: " + entry.getName());
			length = entry.getSize();
			iStream = zipStream;
			break;
		    }
		}
		
		if (iStream == null) {
		    zipStream.close();
		    return new Response( HTTP_NOTFOUND, MIME_PLAINTEXT,
					 "Error 404, file not found " + filename + " in zi7: " + bucketFilename);		    
		}
	    }  catch (IOException exn2) {
		return new Response( HTTP_NOTFOUND, MIME_PLAINTEXT,
				     "Error 404, zi7 error in: " + bucketFilename+" : " + exn2);
	    }
	}

	// Get MIME type from file name extension, if possible
	String mime = null;
	int dot = uri.lastIndexOf( '.' );
	if ( dot >= 0 )
	    mime = (String)theMimeTypes.get(uri.substring( dot + 1 ).toLowerCase());
	if ( mime == null )
	    mime = MIME_DEFAULT_BINARY;
	
	// Support (simple) skipping:
	long startFrom = 0;
	String range = header.getProperty( "Range" );
	if ( range != null )
	    {
		if ( range.startsWith( "bytes=" ))
		    {
			range = range.substring( "bytes=".length());
			int minus = range.indexOf( '-' );
			if ( minus > 0 )
			    range = range.substring( 0, minus );
			try	{
			    startFrom = Long.parseLong( range );
			}
			catch ( NumberFormatException nfe ) {}
		    }
	    }
	try {
	    iStream.skip( startFrom );
	} catch (IOException exn) {
	    return new Response( HTTP_NOTFOUND, MIME_PLAINTEXT,
				 "Error 404, error in skipping in" + uri + ": " + exn);   
	}

	Response r = new Response( HTTP_OK, mime, iStream );
	r.addHeader( "Content-length", "" + (length - startFrom));
	if (startFrom > 0) {
	    r.addHeader( "Content-range", "" + startFrom + "-" +
			 (length-1) + "/" + length);
	}
	r.addHeader("Cache-Control", "public, max-age=2592000");
	return r;
    }

    public static void main(String[] args) {
	System.out.println("start");
	try {	
	    new ManyZipFileServer(args[0], args[1], Integer.parseInt(args[2]), 8081); }
	catch (IOException exn) {
	    System.err.println(exn);
	}
    }
}

